#!/usr/bin/perl -w

# Copyright 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Usage: perl make-oeis-samples.pl filename filename...
#
# Read the given files and find all the OEIS A-numbers which appear as
# A123456_samples in inline test code.  Print to stdout some Pari/GP code
# which defines those as vectors of the sequence sample values from OEIS
# "stripped" file.  The stripped file should be in ~/OEIS/stripped as per
# Math::OEIS::Stripped.
#
# Usually the Pari/GP code output should be directed to a file where test
# code can read it.  The A-numbers output are sorted and each appears just
# once even if used several times in the input files.
#


use 5.006; # for 3-arg open
use strict;
use warnings;
use Math::OEIS::Stripped;

use FindBin;
use lib $FindBin::Bin;


print "\\\\ Generated by $FindBin::Script -- DO NOT EDIT\n";

my %want_anums;
foreach my $filename (@ARGV) {
  open my $fh, '<', $filename or die "Cannot open $filename: $!\n";
  while (my $line = readline $fh) {
    while ($line =~ /\b(A\d{6,7})_samples\b/g) {
      $want_anums{$1} ||= "$filename:$.:";
    }
  }
  close $fh or die $!;
}

my $num_anums = scalar(keys %want_anums);
print "\\\\ Total $num_anums A-numbers\n";
print "\\\\\n";
print "\\\\ Extracted from the OEIS \"stripped\" file, http://oeis.org/allfiles.html\n";
print "\\\\ See http://oeis.org/wiki/The_OEIS_End-User_License_Agreement\n";
print "\\\\ for terms of use (at the time of writing Creative Commons\n";
print "\\\\ Attribution Non-Commercial 3.0)\n";
print "\n";

my @want_anums = keys %want_anums;
@want_anums = sort {substr($a,1) <=> substr($b,1)} @want_anums;

foreach my $anum (sort keys %want_anums) {
  my $str = Math::OEIS::Stripped->anum_to_values_str($anum);

  # or read from individual A-number files
  if (! defined $str) {
    require Math::NumSeq::OEIS;
    my $seq;
    if (eval { $seq = Math::NumSeq::OEIS->new (anum => $anum) }) {
      $str = '';
      my $join = '';
      while (my ($i,$value) = $seq->next) {
        last if length($str) > 180;
        $str .= $join . $value;
        $join = ',';
      }
    }
  }

  if (! defined $str) {
    my $location = $want_anums{$anum};
    print STDERR "$location values not found for $anum\n";
    exit 1;
  }

  $str =~ s/,+$//;
  print "${anum}_samples = [$str];\n";
}
exit 0;
